cloud = {}
cloud.__index = cloud 

function cloud.new(x, y)
    local instance = setmetatable({}, cloud)
    instance.x = x 
    instance.y = y 
    instance.width = 9
    instance.height = 4
    instance.tile_width = 3
    instance.tile_height = 3
    instance.speed = love.math.random(20, 50)
    instance.xVel = 0 
    instance.table = {}
    for y=1, instance.height do     
        instance.table[y] = {}
        for x=1, instance.width do 
            instance.table[y][x] = {}
        end
    end
    for y, xs in ipairs(instance.table) do 
        for x, value in ipairs(xs) do 
            instance.table[1][x] = love.math.random(0, 1)
            instance.table[2][x] = 1
            instance.table[3][x] = 1
            instance.table[4][x] = love.math.random(0, 1)
        end
    end
    return instance
end

function cloud:update(dt)
    self:move(dt)
    self:wrapAround()
end

function cloud:draw()
    love.graphics.push()
    love.graphics.setColor(1, 1, 1, 0.8)
    for y, xs in ipairs(self.table) do 
        for x, value in ipairs(xs) do 
            if value == 1 then 
                love.graphics.rectangle("fill", self.x + (x-1)*self.tile_width, self.y + (y-1)*self.tile_height, self.tile_width, self.tile_height)
            end
        end
    end
    love.graphics.pop()
end 

function cloud:move(dt)
    self.x = self.x + self.xVel
    self.xVel = self.speed * dt
end

function cloud:wrapAround() 
    if self.x > virtual_width then 
        self.table = {}
        for y=1, self.height do     
            self.table[y] = {}
            for x=1, self.width do 
                self.table[y][x] = {}
            end
        end
        for y, xs in ipairs(self.table) do 
            for x, value in ipairs(xs) do 
                self.table[1][x] = love.math.random(0, 1)
                self.table[2][x] = 1
                self.table[3][x] = 1
                self.table[4][x] = love.math.random(0, 1)
            end
        end
        self.x = -9*self.tile_width
        self.y = love.math.random(0, virtual_height / self.tile_height)
        self.speed = love.math.random(20, 50)
    end
end
