Player = {}
Player.__index = Player 

function Player.new(x, y)
    local instance = setmetatable({}, Player)
    instance.x = x
    instance.y = y
    instance.width = 16
    instance.height = 16

    instance.xVel = 0 
    instance.yVel = 0 
    instance.acc = 400
    instance.friction = 4000
    instance.gravity = 200
    instance.jumpheight = -160
    instance.maxSpeed = 200
    instance.grounded = false
    instance.body = love.physics.newBody(world, instance.x - instance.width / 2, instance.y - instance.height / 2, "dynamic")
    instance.shape = love.physics.newRectangleShape(instance.width, instance.height)
    instance.fixture = love.physics.newFixture(instance.body, instance.shape)
    instance.body:setFixedRotation(true)
    return instance
end

function Player:update(dt)
    self:move(dt)
    self:applyGravity(dt)
end



function Player:draw()
    love.graphics.setColor(1, 1, 1)
    love.graphics.rectangle("line", self.x - self.width / 2, self.y - self.height / 2, self.width, self.height)
    love.graphics.setColor(0, 0, 0)
end

function Player:applyGravity(dt) 
    if not self.grounded then 
        self.yVel = self.yVel + self.gravity * dt
    else 
        self.yVel = 0
    end 
end

function Player:jump(key)
    if key == "space" and self.grounded then
        self.yVel = self.jumpheight
        self.grounded = false
        if self.xVel > 0 then 
            self.xVel = self.xVel / 2
        elseif self.xVel < 0 then
            self.xVel = self.xVel * 2
        end
    end
end


function Player:ground(coll)
    self.cColl = coll
    self.yVel = 0
    self.grounded = true
end
function Player:move(dt)
    self.body:setLinearVelocity(self.xVel, self.yVel, 0)
    self.x, self.y = self.body:getPosition()
    if love.keyboard.isDown("a") and self.xVel > -self.maxSpeed then 
        if self.xVel - self.acc * dt > -self.maxSpeed then 
            self.xVel = self.xVel - self.acc * dt 
        end
    
    elseif love.keyboard.isDown("d") and self.xVel < self.maxSpeed then 
        if self.xVel + self.acc * dt < self.maxSpeed then 
            self.xVel = self.xVel + self.acc * dt
        end
    else
        if self.xVel > 0 then 
            if self.xVel - self.friction * dt > 0 then
                self.xVel = self.xVel - self.friction * dt
            else 
                self.xVel = 0
            end
        elseif self.xVel < 0  then
            if self.xVel + self.friction * dt < 0 then
                self.xVel = self.xVel + self.friction * dt
            else 
                self.xVel = 0
            end
        end
    end
end

function Player:beginCon(a, b, collision)
    if self.grounded then return end
    local mx, my = collision:getNormal()
    if a == self.fixture then
        if my > 0 then 
            self:ground(collision)
        end
    elseif  b == self.fixture then 
        if my < 0 then 
            self:ground(collision)
        end
    end
end

function Player:endCon(a, b, collision)
    if a == self.fixture or b == self.fixture then
        if self.cColl == collision then 
            self.grounded = false
        end
    end
end

